;; Shoe Box Music
;;
;; Copyright 2017 - By Michael Kohn
;; http://www.mikekohn.net/
;; mike@mikekohn.net
;;
;; Control some servos and LED's and digital music with a coin acceptor
;; using an MSP430F5529

.msp430x

.include "msp430f5529.inc"

;  r4 = delay count
;  r5 = dance count
;  r6 = delay outer loop
;  r7 = led values
;  r8 =
;  r9 = temp in interrupt
; r10 = increments on every Timer A interrupt
; r11 = point to next byte in sound buffer
; r12 =
; r13 = temp
; r14 = temp
; r15 = temp

; 4MHz * 0.020 = 80000 cycles or 220 interupts
; 4MHz * 0.001 =  4000 cycles or  11 interrupts
; 4MHz * 0.002 =  8000 cycles or  22 interrupts

RAM equ 0x2400
SERVO_0 equ RAM+0
SERVO_1 equ RAM+2
SERVO_2 equ RAM+4
SERVO_3 equ RAM+6

.org 0x4400
start:
  ;; Turn off watchdog
  mov.w #0x5a80, &WDTCTL

  ;; Interrupts off
  dint

  ;; Set up stack pointer
  mov.w #0x4400, SP

  ;; Set up XT2 (4MHz)
  bis.b #0x0c, &P5SEL
  mov.w #SELS_5|SELM_5|SELA_2, &UCSCTL4
  bic.w #SELREF_7, &UCSCTL3
  bis.w #SELREF_2, &UCSCTL3
  mov.w #0, &UCSCTL0
  bic.w #XT2OFF, &UCSCTL6
wait_clock:
  bic.w #XT2OFFG|XT1LFOFFG|XT1HFOFFG|DCOFFG, &UCSCTL7
  bic.w #OFIFG, &SFRIFG1
  mov.w #5000, r15
dec_again:
  dec r15
  jnz dec_again
  bit.w #OFIFG, &SFRIFG1
  jnz wait_clock
  mov.w #0, &UCSCTL5

  ;; Set up output pins
  ;; P1.0 = LED1
  ;; P1.2 = Shoe box LED0
  ;; P1.3 = Shoe box LED1
  ;; P1.4 = Shoe box LED2
  ;; P3.0 = SIMO 
  ;; P3.2 = CLK
  ;; P3.3 = TX
  ;; P3.4 = RX
  ;; P4.7 = LED2
  ;; P6.0 = servo 0 (top)
  ;; P6.1 = servo 1 (bottom)
  ;; P6.2 = servo 2 (right)
  ;; P6.3 = servo 3 (left)
  ;; P6.5 = coin input
  ;; P6.6 = /CS
  mov.b #0x1f, &P1DIR
  mov.b #0x00, &P1OUT
  mov.b #0x1f, &P3SEL
  mov.b #0x80, &P4DIR
  mov.b #0x80, &P4OUT
  mov.b #0x4f, &P6DIR
  mov.b #0x40, &P6OUT

  ;; Set up SPI
  mov.b #UCSWRST, &UCB0CTL1            ; set reset
  bis.b #UCSSEL_2|UCSWRST, &UCB0CTL1   ; SMCLK
  mov.b #UCCKPH|UCMSB|UCMST|UCSYNC, &UCB0CTL0 ; MSB first, Master, 3 pin SPI, Sync mode
  mov.b #1, &UCB0BR0
  mov.b #0, &UCB0BR1
  bic.b #UCSWRST, &UCB0CTL1            ; clear reset

  ;; Set up Timer A
  ;; Need ~11000 samples a second
  mov.w #363, &TA0CCR0
  mov.w #(TASSEL_2|MC_1), &TA0CTL ; SMCLK, DIV1, COUNT to TACCR0
  mov.w #CCIE, &TA0CCTL0
  mov.w #0, &TA0CCTL1

  mova #0, r11

  mov.w #12, &SERVO_0
  mov.w #19, &SERVO_1
  mov.w #21, &SERVO_2
  mov.w #12, &SERVO_3

  mov.w #0, r5

  ;; Interrupts on
  eint

  ;; Setup UART
  mov.b #UCSSEL_2|UCSWRST, &UCA0CTL1
  mov.b #0, &UCA0CTL0
  mov.b &9600, &UCA0BR0
  mov.b &9600, &UCA0BR1
  bic.b #UCSWRST, &UCA0CTL1

  mov.w #0x3fff, r9
  call #update_dac

main:
  bit.b #0x20, &P6IN
  jeq main

  mov.w #0, r5
  mov.b #4, r7
  mov.w #19, &SERVO_0
  mova #sound_start, r11

dance_repeat:
  call #toggle_and_delay
  cmp.w #3, r5
  jge stop_dancing
  mov.w #21, &SERVO_1
  mov.w #10, &SERVO_2
  mov.w #22, &SERVO_3
  call #toggle_and_delay
  cmp.w #3, r5
  jge stop_dancing
  mov.w #17, &SERVO_1
  mov.w #22, &SERVO_2
  mov.w #10, &SERVO_3
  jmp dance_repeat

stop_dancing:
  mova #0, r11
  mov.w #12, &SERVO_0
  mov.w #19, &SERVO_1
  mov.w #21, &SERVO_2
  mov.w #12, &SERVO_3

  bic.b #0x1c, &P1OUT
  mov.w #0x3fff, r9
  call #update_dac
  jmp main

toggle_and_delay:
repeat:
  bic.b #0x1c, &P1OUT
  bis.b r7, &P1OUT

  mov.w #10, r6
delay_loop_outer:
  mov.w #0, r4
delay_loop:
  dec.w r4
  jnz delay_loop
  dec.w r6
  jnz delay_loop_outer
  rla r7
  cmp.b #0x20, r7
  jnz no_led_reset
  mov.b #4, r7
no_led_reset:
  ret

timer_interrupt_a:
  inc.w r10

  cmp.w #220, r10
  jne skip_servo
  bic.b #0x0f, &P6OUT
  mov.w #0, r10
skip_servo:

  cmp.w r10, &SERVO_0
  jnz not_servo_0
  bis.b #0x01, &P6OUT
not_servo_0:
  cmp.w r10, &SERVO_1
  jnz not_servo_1
  bis.b #0x02, &P6OUT
not_servo_1:
  cmp.w r10, &SERVO_2
  jnz not_servo_2
  bis.b #0x04, &P6OUT
not_servo_2:
  cmp.w r10, &SERVO_3
  jnz not_servo_3
  bis.b #0x08, &P6OUT
not_servo_3:

  cmpa #0, r11
  jeq exit_interrupt

  mov.b @r11, r9
  rla.w r9
  rla.w r9
  rla.w r9
  rla.w r9
  bis.w #0x3000, r9              ; add SPI command nibble

  call #update_dac

  ;; inc sound buffer pointer and loop if needed
  incx.a r11
  cmpa #sound_stop, r11
  jne sound_buffer_no_loop
  mova #sound_start, r11
  inc.w r5
sound_buffer_no_loop:
exit_interrupt:
  reti

update_dac:
  ;; Send word over SPI
  bic.b #0x40, &P6OUT            ; /CS = 0 (enable)

skip_load:
  swpb r9
  bic.b #UCRXIFG, &UCB0IFG
  mov.b r9, &UCB0TXBUF
  swpb r9
spi_hibyte_wait:
  bit.b #UCRXIFG, &UCB0IFG
  jz spi_hibyte_wait

  bic.b #UCRXIFG, &UCB0IFG
  mov.b r9, &UCB0TXBUF
spi_lobyte_wait:
  bit.b #UCRXIFG, &UCB0IFG
  jz spi_lobyte_wait

  bis.b #0x40, &P6OUT            ; /CS = 1 (disable)
  ret

.org 0x14400
sound_start:
  ;db 0, 0, 0, 0, 0, 255, 255, 255, 255, 255
  .binfile "out.bin"
  ;.binfile "java_genesis.bin"
sound_stop:

.org 0xffe8
vectors:
.org 0xffea
  dw timer_interrupt_a     ; Timer0_A3 TACCR0, CCIFG
.org 0xfffe
  dw start                 ; Reset

